;Rev counter 

;Now called DRO_Tacho_V04
; 25/05/2014 Renamed to Tacho_V03  Changing to yust outputting three bytes od raw binary data
;              Removing BCD conversion code.



;Measure period of one revolution (Or one pulse if multiple pulses per revolution)


;
;Interrupt routine changed to handle more than one interrupt while in the routine.
;

; Now wait until about 500 mS has elapsed since start of capture time. Keep track of number of CCP1 interrupts 
;since start of capture. Do calculation on first CCP1 interrupt that the accumulated count exceeds 500 mS worth of counts.
; As one count is 0.4 us then 500 mS = 500000/0.4 = 1250000  ( = 0x1312D0 )



; Need to be able to count to more than 255 CCP1 interupts to allow higher input frequency in multi pulse per rev version
; To do this two new variables are required CCP1_count_H and CCP1_count_copy_H
; CCP1_count will be renamed CCP1_count_L and CCP1_count_copy will be renamed CCP1_count_copy_L





;Using 20 Mhz xtal system clock is 5 Mhz   (200ns)


	
;PIC16f628A, set for 20.00MHz XTAL, WDT OFF, POR on
;Instruction time at 20MHZ is 0.2 us




	list P=16f628
	#include "p16f628a.inc"
	__config _HS_OSC & _WDT_OFF & _PWRTE_ON & _LVP_OFF

;written in MASM

;				---- Constants ---
Timeout	EQU d'115' ;(gives about 3 seconds			;Maximum time allowed between input pulses 
;					Timeout *  26.2144 ms = maximum time allowed between input pulses
;					;Say 3 seconds This will allow a minimum speed of 20 RPM
;Update_Time	EQU d'50'	;Number of 5 ms between display updates (250 ms)
Update_Time	EQU d'100'	;Number of 5 ms between display updates (500 ms)

BAUD_CONSTANT       EQU 40            ;Constant for baud generator for 19200 baud
                                    ;Fosc is 20MHz



	CBLOCK	0x20	;GPR' starts at 0x20

Save_STATUS		;Save during interrupt
Save_W			;Save during interrupt
Save_PCLATH		;Save during interrupt
Save_FSR		;Save during interrupt	

Mux_time		;Time between digits
Timeout_Counter			;Used to test if maximum time allowed between input pulses has been exceeded

TMR1_Int_CountL
TMR1_Int_CountH

Captured_count0		;LSB
Captured_count1
Captured_count2
Captured_count3		;MSB


T0_Count0		;LSB
T0_Count1
T0_Count2
T0_Count3		;MSB


Accumulated_Count0		;LSB
Accumulated_Count1
Accumulated_Count2
Accumulated_Count3		;MSB

CCP1_count_L			;Number of CCP1 interrupts since start of capture (T0_time)
CCP1_count_H
CCP1_count_copy_L			;Value of CCP1_count when time reached to update display
CCP1_count_copy_H

Capture_State			;
;				0	Waiting for first CCP1 interrupt
;				1 	Waiting for next CCP1 interrupt
;






Flags
Flags2			;Used for testing to see when TMR1 and CCP1 interrupts occure at almost the same time
Flags3			;Bit 0 set when in setup mode



;				Registers used by maths routines
REGA0				;lsb
REGA1
REGA2
REGA3				;msb

REGB0				;lsb
REGB1
REGB2
REGB3				;msb

REGC0				;lsb
REGC1
REGC2
REGC3				;msb

;MTEMP
MCOUNT

;		Registers used by EEPROM ROUTINES

DATA_EE_ADDR
DATA_EE_DATA


	ENDC			;Must end at 07F

;LED_Port	EQU PORTB
;Dig_Port	EQU PORTA


;					-----

;		Flag bit usage
;		Bit 0		Start of next digit time
;Flags
	#define Next_Dig_Flag Flags, 0	;Flag to indicate start of next digit time
	#define Display_Update_Flag Flags, 1	;Flag to indicate display update required (Set in interrupt routine. )
	#define CCP1_Flag Flags, 2
	#define Div_Err_Flag Flags, 3
	#define LZB_Flag Flags, 4		;Flag for leading zero blanking

;Flags3
	#define Setup_Flag Flags3,0
	#define	PPR_Change Flags3,1	;Button has been pressed so new PPR value needs saving to EEPROM


;Flags2
;	Bit 0	CCP1 interrupt
;	Bit 1	TMR1 interrupt
;	Bit 2	TMR0 interrupt


;==========================================================================
;
;I/O Pin useage

;	RA0		Output		Digit drive bit 4 LSB		NOT used now
;	RA1		Output		Digit drive bit 3		NOT used now
;	RA2		Output		Digit drive bit 2		NOT used now
;	RA3		Output		Digit drive bit 1 MSB	(Pin 2)		NOT used now
;	RA4/TOCK1	input		Push button to set counts per rev		(Pin 3)		NOT used now

;	RB1/RX		input		USART RX Data  (Pin 7)
;	RB2/TX		output		USART TX Data  (Pin 8)

;	RB3		Input	  Tacho input	         (Pin 9)  CCP1 Input from sensor



;					--------------------------

;			*** Define 7 Segment patterns   (Stripboard version) (NOTE active low.)

;			Assignment of PORTA bits to display segments
;			(These can be changed around to make track layout on PCB easier.)



	

;			Assignment of PORTB bits to digit positions
;			(These can be changed around to make track layout on PCB easier.)
;	#define Dig1	3
;	#define Dig2	2
;	#define Dig3	1
;	#define Dig4	0

;					---------------------------


; define reset and interrupt vector start addresses

	org	0	  	; start at address 0000h
	goto	INIT		; normal service routines from Reset vector
	org     4		; interrupt vector 0004h, start interrupt routine here
	goto	Interupt

; **********************************************************************************************



;	Initialize 
;
        ORG 0x0005      	;Start of program memory


    	DE      "DRO_Tacho_03 25/05/14"
;
INIT    BCF INTCON,GIE  		;turn off global interrupts
        BTFSC INTCON,GIE 	;Confirm global interrupts off
        GOTO INIT

	MOVLW 	0x07        	; activate PORTA for PC16F628 as digital
	MOVWF	CMCON

	BSF 	STATUS,5	;Select register bank 1
        MOVLW	B'00001010'
        MOVWF	TRISB     	;Configure port B0, B2 and B4 - B7 as outputs. ports B1 & B3 as input 
        MOVLW	B'00010000'
        MOVWF	TRISA     	;Port A0-A3 as output A4 as input
;

        MOVLW	B'10001000' 	; Weak pull-up on PORTB disabled (bit7=1) TMR0 prescaler assigned to watchdog  (Overflow every 51,2 us)
        MOVWF	OPTION_REG


;Configure TIMER1						(Used to time /count input pulses.)

	MOVLW	b'00000001'	;Enable timer1 interrupt
	MOVWF	PIE1

	BCF	STATUS,5	;Select register bank 0

	MOVLW	b'00010001'	;Configure timer 1
	MOVWF	T1CON		;Prescaler 1:2, Internal clock, Timer on  (Increment every 0.4 us)


;Configure UART

    	bsf     STATUS,RP0    ;select bank 1

    	movlw   BAUD_CONSTANT    ;set baud rate 
    	movwf   SPBRG
    	bsf     TXSTA,BRGH    ;baud rate high speed option
    	bsf     TXSTA,TXEN    ;enable transmission
;	BCF	TXSTA, SYNC		

    	bcf     STATUS,RP0        ;select bank 0
    	bsf     RCSTA,CREN    ;enable reception

    	bsf     RCSTA,SPEN    ;enable serial port






Not_Setup
;Configure CCP1
	MOVLW	b'00000100'	;Capture mode, every falling edge
	MOVWF	CCP1CON

	BSF	STATUS, 5	;Select bank 1
	BSF	PIE1, CCP1IE	;Eneble CCP1 interrupt
	BCF	STATUS, 5	;Select bank 0

	CLRF	Capture_State

;Set initial display to read zero

	
	CLRF	TMR1L		;Clear timer 1
	CLRF	TMR1H

	BCF PIR1, 0x2  		;Clear CCP1IF

	BCF	Display_Update_Flag


;turn on global interrupts

	MOVLW	B'11000000'		;Global interrupts, Peripheral interrupts
  	MOVWF 	INTCON  		;turn on required interrupts

;					---------  End if initialisation ---------------

	GOTO	MAIN

; *******************************************************************************
; * Interupt handler
; *
; *
 ;*******************************************************************************

Interupt

;Save various registers

	MOVWF Save_W         	;Save W                            
	MOVF STATUS, W                         
	CLRF STATUS  		;Bank 0 will be selected
	MOVWF	Save_STATUS
	MOVF	PCLATH, W
	MOVWF	Save_PCLATH
	MOVF	FSR, W
	MOVWF	Save_FSR

	CLRF	Flags2


;	BTFSS	INTCON, T0IF
	GOTO	Test_CCP1_Int	;Not Timer 0 interrupt

;				Timer0 used for display multiplexing and display update timming

Timer0_Int		;(Interrupts every 51.2 uS)   (Need digit time of about 5 mS) (This would need divide by about 97)


TMR0_End
	BCF INTCON, T0IF 	;Clear interrupt flag
;					-------------------


Test_CCP1_Int

	BTFSS	PIR1, CCP1IF
	GOTO	Test_Timer1_Int		;Not CCP1 interrupt
	BSF	Flags2, 0

	INCF	CCP1_count_L,F		;Increment count (Low byte) of CCP1 interrupts since T0 time
	BTFSC 	STATUS, Z 
	INCF	CCP1_count_H,F		;Increment count (High byte) of CCP1 interrupts since T0 time Z set


	MOVF	Capture_State, F	;Just to test if zero
	BTFSS	STATUS, Z
	GOTO	Not_T0
	INCF	Capture_State, F	;Increment to state 1

;				Copy count at last T0 time to T0_Count
	MOVF	Captured_count0, W	;LSB
	MOVWF	T0_Count0
	MOVF	Captured_count1, W
	MOVWF	T0_Count1	
	MOVF	Captured_count2, W
	MOVWF	T0_Count2
	MOVF	Captured_count3, W	;MSB
	MOVWF	T0_Count3


Not_T0
;		
;				Copy CCPR1L & H to Captured_count0 & 1
	MOVF	CCPR1L, W
	MOVWF	Captured_count0
	MOVF	CCPR1H, W
	MOVWF	Captured_count1

	MOVF	TMR1_Int_CountL, W
	MOVWF	Captured_count2
	MOVF	TMR1_Int_CountH, W
	MOVWF	Captured_count3


	BTFSS PIR1, 0        	;TMR1IF     (Has TMR1 interrupt also occured ?) 
	GOTO	CCP1_Int2
        BTFSC Captured_count1, 0x7		;Top bit of  CCPR1H      Capture occured after overflow  if clear
        GOTO CCP1_Int2		;  (The capture TMR1_Int_Count occured befor TMR1 interrupt)         
	INCF Captured_count2, F     		;Increment copy of TMR1_Int_Count  if bit 7 of CCPR1H NOT set                  
	BTFSC STATUS, Z                      
	INCF Captured_count3, F  

CCP1_Int2
;			------ Now calculate number of counts since T0 Time CCP interrupt ----


	MOVF	Captured_count0, W	
	MOVWF	Accumulated_Count0
	MOVF	Captured_count1, W	
	MOVWF	Accumulated_Count1
	MOVF	Captured_count2, W	
	MOVWF	Accumulated_Count2
	MOVF	Captured_count3, W	
	MOVWF	Accumulated_Count3


	MOVF	T0_Count0, W		;Subtract T0 count from Captured_count (Accumulated_Count)
	SUBWF	Accumulated_Count0, F
	MOVF	T0_Count1, W
	BTFSS	STATUS, C     	;Carry set for zero or positive
	INCFSZ	T0_Count1, W
	SUBWF	Accumulated_Count1, F
	MOVF	T0_Count2, W
	BTFSS	STATUS, C
	INCFSZ	T0_Count2, W
	SUBWF	Accumulated_Count2, F
	MOVF	T0_Count3, W
	BTFSS	STATUS, C
	INCFSZ	T0_Count3, W
	SUBWF	Accumulated_Count3, F
	BCF	STATUS, C

;								Note timer 1 increments every 0.4 us
;	Now test if accumulated count has exceeded 1250000  ( = 0x1312D0 ) (Counts in 500 mS)
; 250 ms = 625000   (= 98968 Hex)

;	May be better to do this in main loop

;				Test if	 Accumulated count     >= 0x1312D0  (Counts in 500 mS)
	MOVLW	0x00
	SUBWF	Accumulated_Count3, W
	BTFSS	STATUS, Z
	GOTO	Accumulated_Test_End
	MOVLW	0x13
	SUBWF	Accumulated_Count2, W
	BTFSS	STATUS, Z
	GOTO	Accumulated_Test_End
	MOVLW	0x12
	SUBWF	Accumulated_Count1, W
	BTFSS	STATUS, Z
	GOTO	Accumulated_Test_End
	MOVLW	0xD0
	SUBWF	Accumulated_Count0, W		;End of test 

Accumulated_Test_End

	BTFSS	STATUS, C	;Carry set if Accumulated_Count >= 0x001312D0  (Carry set if result zero or positive)
	GOTO	Not_time	;
	
;Set things ready for start of next timing period
	CLRF	Capture_State
;Save the number of pulses that occured in the timing period
	MOVF	CCP1_count_L, W
	MOVWF	CCP1_count_copy_L
	MOVF	CCP1_count_H, W
	MOVWF	CCP1_count_copy_H

;Set the pulse count back to zero
	CLRF	CCP1_count_L
	CLRF	CCP1_count_H



	BSF	Display_Update_Flag  ;New value to be displayed

Not_time
	MOVLW	Timeout			;Re load Timeout counter
	MOVWF	Timeout_Counter
                         
	BCF PIR1, 0x2  		;CCP1IF

;					--- End of CCP1 interrupt handler ---



;				Timer1 interupt handler 
;				Should come here every (1.6 us * 6553) =  104857.6uS= 104.8576 ms (This value not used now.)
Test_Timer1_Int
	BTFSS	PIR1,TMR1IF	;TMR1 interupt Interrupts every 0.4 * 256 * 256 uS = 26214.4 uS = 26.2144  mS
	GOTO	Int_End

	INCF	TMR1_Int_CountL,F
	BTFSC	STATUS, Z	;If overflow then increment Byte 3
	INCF	TMR1_Int_CountH,F

	MOVF	Timeout_Counter, F	; Just to test if it is zero (This is to deal with no input pulses)
	BTFSS	STATUS, Z		; This counter is reloaded every CCP1 interrupt
	DECF	Timeout_Counter, F
	
	BSF	Flags2, 1	
	BCF PIR1, TMR1IF 

Int_End				;Leave interrupt routine here


;Restore registers 

	MOVF	Save_FSR, W
	MOVWF	FSR
	MOVF	Save_PCLATH, W
	MOVWF	PCLATH	
	MOVF Save_STATUS, W                           
	MOVWF STATUS      ;Restore STATUS                     
	SWAPF Save_W, F                          
	SWAPF Save_W, W     ;Restore W    
	RETFIE

;						*********** End of interrupt routine **************
;
;
;			---------------------------------------------------------


MAIN				;Main program loop Just loop waiting for interupt.




	;TEST CODE 
;	BSF	PORTB, 7
;	MOVLW	0x0D		;C/R
;	BCF	PORTB, 7
;	CALL	SerialTransmit

;	GOTO MAIN



	BTFSC	Display_Update_Flag
	CALL	Update_Display		; Calculate RPM value and output it to USART TX

;	Test for timeout   (No pulses for more than about three seconds)
	MOVF	Timeout_Counter, F	; Just to test if it is zero
	BTFSS	STATUS, Z
	GOTO	No_Clear

	;Force value displayed to zero as there have been no input pulses for timeout period



No_Clear
	GOTO	MAIN

;



;	***************************************************************************************************


	
;				+++++++++++++++ Subroutines from here +++++++++++++++++++++++++


; *****************************************************************************
; * Function 
; *	
; *
; *****************************************************************************

 
Update_Display

	BCF	INTCON, GIE	;Disable all interrupts during copy

	MOVF	Accumulated_Count0, W	;LSB
	MOVWF	REGA0
	MOVF	Accumulated_Count1, W
	MOVWF	REGA1	
	MOVF	Accumulated_Count2, W
	MOVWF	REGA2
	MOVF	Accumulated_Count3, W	;MSB
	MOVWF	REGA3

	MOVF	CCP1_count_copy_L, W
	MOVWF	REGB0
	MOVF	CCP1_count_copy_H, W
	MOVWF	REGB1

	BSF	INTCON, GIE	;Re enable interrupts

;	CLRF	REGB1
	CLRF	REGB2
	CLRF	REGB3


;				Divide accumulated counts by number of CCP1 interrupts

	CALL	divide		;Result will be in REGA

	MOVF	REGA0, W	;LSB
	MOVWF	REGB0
	MOVF	REGA1, W
	MOVWF	REGB1	
	MOVF	REGA2, W
	MOVWF	REGB2
	MOVF	REGA3, W	;MSB
	MOVWF	REGB3

;		Load value into dividend   150000000 in HEX is 8F0D180 (This value fo 1 pulse per rev.)


	MOVLW	0x08
	MOVWF	REGA3

	MOVLW	0xF0
	MOVWF	REGA2

	MOVLW	0xD1
	MOVWF	REGA1

	MOVLW	0x80
	MOVWF	REGA0

	CALL	divide		;Divide number of counts (0.4 uS) for 1 rev (1 CCP1 interrupt) into 150000000   (Number of 0.4 uS in 1 minute)


;	Result will be in REGA0 - REGA3   (This wiil be the value of the number of pulses in one minute)

;	********************************************************************************************************

;			Now need to test that reading is not greater than 9999 decimal = 0x270F


;	MOVF	REGA3, W	;These two must be zero
;	IORWF	REGA2, W
;	BTFSS	STATUS, Z
;	GOTO 	Oflow

;	MOVF	REGA1, W
;	SUBLW	0x26
;	BTFSC	STATUS, C 	;Carry set for zero or positive
;	GOTO 	No_Oflow

;	MOVF	REGA0, W
;	SUBLW	0x0F
;	BTFSS	STATUS, C 	;Carry set for zero or positive
;	GOTO 	Oflow

No_Oflow
;	MOVF	REGA2, W
;	MOVWF	rBin0		;MSB
;	MOVF	REGA1, W
;	MOVWF	rBin1
;	MOVF	REGA0, W
;	MOVWF	rBin2		;LSB

;	CALL	Hex_Serial_Out
	CALL	Binary_Out

;	CALL	BintoBCD


; Result will be in  rBCD to rBCD3

; Now output data to USART TX 



	BCF	Display_Update_Flag	; Clear Display_Update_Flag
	RETURN



Oflow

	BCF	Display_Update_Flag
	RETURN

;	-----------------------------------------------------------------------------------------
;



; ***************************************************************************************
; *											*
; * Binary_Out        Output 24 bit binary value as three bytes to serial port.		*
; * value to be output in REGA0 - REGA2  (REGA2 is MSB)					*
; *											*
; ***************************************************************************************

Binary_Out

	MOVF	REGA0, W
	CALL	SerialTransmit
	MOVF	REGA1, W
	CALL	SerialTransmit
	MOVF	REGA2, W
	CALL	SerialTransmit
	RETURN


;*****************************************************************************        
;
;   Function :  SerialTransmit
;               This function sends the byte in W over the RS232 port. The 
;               function will wait until previous data has been sent
;
;   Input:      Byte in W
;
;   Output:     
;
;*****************************************************************************        
SerialTransmit
;	BSF	PORTB, 6		;TEST
	NOP
	NOP
	NOP
;	BCF	PORTB, 6		;TEST
    	btfss   PIR1,TXIF    ;check that buffer is empty
    	goto    SerialTransmit	;$-1
    	movwf   TXREG        ;transmit byte
    	return




;				**** Maths subroutines ****

;	*********************************************************************************************************
;	*													*
;	*				32 Bit divide								*
;	*													*
;	*	Dividend in REGA0, REGA1, REGA2, REGA3	(REGA3 most significant.)				*
;	*	Divisor in REGB0, REGB1, REGB2, REGB3	(REGB3 most significant.)				*
;	*	Result in REGA0, REGA1, REGA2, REGA3	(REGA3 most significant.)				*
;	*	Remaindor in REGC0, REGC1, REGC2, REGC3	(REGC3 most significant.)				*
;	*	Carry set on return if overflow or division by zero						*
;	*													*
;	*********************************************************************************************************
;*** 32 BIT DIVIDE ***
;REGA / REGB -> REGA
;Remainder in REGC
;Return carry set if overflow or division by zero

divide
	MOVF	REGB0, W		;Check for division by zero
	IORWF	REGB1, W
	IORWF	REGB2, W
	IORWF	REGB3, W
	SUBLW	0
	BTFSC	STATUS, C	;Carry set if result is zero or positive
	GOTO	Divide_Exit	;return			;Overflow  (IE divide by zero) (Carry will be set)


	CLRF	REGC0		;Clear remainder
	CLRF	REGC1
	CLRF	REGC2
	CLRF	REGC3

	MOVLW	D'32'		;Loop counter
	MOVWF	MCOUNT

divloop	
	RLF	REGA0,f		;LSB	;Shift dividend (REGA) msb into remainder (REGC)
	RLF	REGA1,f
	RLF	REGA2,f
	RLF	REGA3,f		;MSB  (Top bit goes to carry)

	RLF	REGC0,f		;Carry goes to bottom bit
	RLF	REGC1,f
	RLF	REGC2,f
	RLF	REGC3,f

	MOVF	REGB3, W	;Test if remainder (REGC) >= divisor (REGB)
	SUBWF	REGC3, W
	BTFSS	STATUS, Z
	GOTO	Div_test_end
	MOVF	REGB2, W
	SUBWF	REGC2, W
	BTFSS	STATUS, Z
	GOTO	Div_test_end
	MOVF	REGB1, W
	SUBWF	REGC1, W
	BTFSS	STATUS, Z
	GOTO	Div_test_end
	MOVF	REGB0, W
	SUBWF	REGC0, W		;End of test 
Div_test_end	
	BTFSS	STATUS, C	;Carry set if remainder >= divisor  (Carry set if result zero or positive)
	GOTO	No_Go		;if remainder not >= divisor (If less than divisor)

	MOVF	REGB0, W		;Subtract divisor (REGB) from remainder (REGC)
	SUBWF	REGC0, F
	MOVF	REGB1, W
	BTFSS	STATUS, C
	INCFSZ	REGB1, W		;
	SUBWF	REGC1, F
	MOVF	REGB2, W
	BTFSS	STATUS, C
	INCFSZ	REGB2, W
	SUBWF	REGC2, F
	MOVF	REGB3, W
	BTFSS	STATUS, C
	INCFSZ	REGB3, W
	SUBWF	REGC3, F		;End of subtraction  Remainder in REGC
	BCF	STATUS, C	;Clear carry
	BSF	REGA0, 0		;Set quotient bit

No_Go	DECFSZ	MCOUNT, F	;Next
	GOTO	divloop		;Loop

	GOTO	Divide_Exit


Divide_Exit
	RETURN

;			--- End of divide ---

;				**** End of maths routines ****



;	*************************************************************************************************


        END            	;final line. This line must be retained.


